"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const log_1 = require("./log");
const messages_1 = require("./messages");
class Matcher {
    static TypeOf(typeOf, cb) {
        return (rxme, sub) => {
            if (typeof (rxme.data) == typeOf) {
                return cb(rxme.data, sub, rxme);
            }
            return false;
        };
    }
    static Number(cb) {
        return this.TypeOf('number', cb);
    }
    static Boolean(cb) {
        return this.TypeOf('boolean', cb);
    }
    static String(cb) {
        return this.TypeOf('string', cb);
    }
    static WildCard(cb) {
        return cb;
    }
    static Error(cb) {
        return (rxme, sub) => {
            if (rxme.data instanceof messages_1.ErrorContainer) {
                return cb(rxme.data.error, sub, rxme);
            }
            return false;
        };
    }
    static Done(cb) {
        return (rxme, sub) => {
            console.log('Done:', rxme);
            if (rxme.data instanceof messages_1.DoneMsg) {
                return cb(rxme.data.result, sub, rxme);
            }
            return false;
        };
    }
    static Complete(cb) {
        return (rxme, sub) => {
            if (rxme.data instanceof messages_1.CompleteMsg) {
                return cb(sub, rxme);
            }
            return false;
        };
    }
    static Observer(cb) {
        return (rxme, sub) => {
            if (rxme.data.next && rxme.data.complete && rxme.data.error) {
                return cb(rxme.data, sub, rxme);
            }
            return false;
        };
    }
    static Type(typ, cb) {
        return (rxme, sub) => {
            if (rxme.data instanceof typ) {
                return cb(rxme.data, sub, rxme);
            }
            return false;
        };
    }
    static ArrayOf(typ, cb) {
        return (rxme, sub) => {
            if (Array.isArray(rxme.data) && (rxme.data.length == 0 || (rxme.data[0] instanceof typ))) {
                return cb(rxme.data, sub, rxme);
            }
            return false;
        };
    }
    static Log(cb) {
        return (rxme, sub) => {
            const ret = rxme.data instanceof log_1.LogEntry;
            if (ret) {
                return cb(rxme.data, sub, rxme);
            }
            return false;
        };
    }
}
exports.Matcher = Matcher;
//# sourceMappingURL=matcher.js.map